@@ -63,7 +63,7 @@ class CSVTable(TableData):
         if self.options.set is None and self.options.param is None:
             raise IOError("Unspecified model component")
         self.FILE = open(self.filename, 'w')
-        table = self.get_table()
+        table = self._get_table()
         writer = csv.writer(self.FILE)
         writer.writerows(table)
         self.FILE.close()
