@@ -2,8 +2,8 @@
 #
 #  Pyomo: Python Optimization Modeling Objects
 #  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
-#  Under the terms of Contract DE-NA0003525 with National Technology and 
-#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  Under the terms of Contract DE-NA0003525 with National Technology and
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
@@ -22,48 +22,13 @@ currdir = dirname(abspath(__file__))+os.sep
 
 import pyutilib.th as unittest
 
-from pyomo.environ import *
+from pyomo.environ import ConcreteModel, AbstractModel, Var, Constraint, \
+    ConstraintList, Param, RangeSet, Set, Expression, value, \
+    simple_constraintlist_rule, simple_constraint_rule
+from pyomo.core.base.constraint import _GeneralConstraintData
 
 from six import StringIO
 
-class LogBuffer(object):
-
-    def __init__(self, logger, logLevel=None):
-        if isinstance(logger, logging.Logger):
-            self.logger = logger
-        else:
-            self.logger = logging.getLogger(logger)
-        self.buffer = StringIO()
-        if logLevel is None:
-            self.old_level = None
-        else:
-            self.old_level = self.logger.getEffectiveLevel()
-            self.logger.setLevel(logLevel)
-
-        self.logHandler = logging.StreamHandler(self.buffer)
-        self.logger.addHandler(self.logHandler)
-
-    def __del__(self):
-        self.close()
-
-    def close(self):
-        if self.logger is None:
-            return
-        if self.old_level is not None:
-            self.logger.setLevel(self.old_level)
-
-        self.logger.removeHandler(self.logHandler)
-        self.logHandler.flush()
-        self.buffer.flush()
-        self.logHandler = None
-        self.logger = None
-
-    def value(self):
-        if self.logHandler is not None:
-            self.logHandler.flush();
-        self.buffer.flush()
-        return self.buffer.getvalue()
-
 
 class TestConstraintCreation(unittest.TestCase):
 
@@ -352,6 +317,47 @@ class TestConstraintCreation(unittest.TestCase):
         model.c = Constraint(rule=rule)
         self.assertRaises(ValueError, model.create_instance)
 
+    def test_expr_construct_invalid(self):
+        m = ConcreteModel()
+        c = Constraint(rule=lambda m: None)
+        self.assertRaisesRegexp(
+            ValueError, ".*rule returned None",
+            m.add_component, 'c', c)
+
+        m = ConcreteModel()
+        c = Constraint([1], rule=lambda m,i: None)
+        self.assertRaisesRegexp(
+            ValueError, ".*rule returned None",
+            m.add_component, 'c', c)
+
+        m = ConcreteModel()
+        c = Constraint(rule=lambda m: True)
+        self.assertRaisesRegexp(
+            ValueError,
+            ".*resolved to a trivial Boolean \(True\).*Constraint\.Feasible",
+            m.add_component, 'c', c)
+
+        m = ConcreteModel()
+        c = Constraint([1], rule=lambda m,i: True)
+        self.assertRaisesRegexp(
+            ValueError,
+            ".*resolved to a trivial Boolean \(True\).*Constraint\.Feasible",
+            m.add_component, 'c', c)
+
+        m = ConcreteModel()
+        c = Constraint(rule=lambda m: False)
+        self.assertRaisesRegexp(
+            ValueError,
+            ".*resolved to a trivial Boolean \(False\).*Constraint\.Infeasible",
+            m.add_component, 'c', c)
+
+        m = ConcreteModel()
+        c = Constraint([1], rule=lambda m,i: False)
+        self.assertRaisesRegexp(
+            ValueError,
+            ".*resolved to a trivial Boolean \(False\).*Constraint\.Infeasible",
+            m.add_component, 'c', c)
+
     def test_nondata_bounds(self):
         model = ConcreteModel()
         model.c = Constraint()
@@ -741,6 +747,20 @@ class TestSimpleCon(unittest.TestCase):
         inst = model.create_instance()
         self.assertEqual(len(inst.c),1)
 
+    def test_setitem(self):
+        m = ConcreteModel()
+        m.x = Var()
+        m.c = Constraint()
+        self.assertEqual(len(m.c), 0)
+
+        m.c = m.x**2 <= 4
+        self.assertEqual(len(m.c), 1)
+        self.assertEqual(list(m.c.keys()), [None])
+        self.assertEqual(m.c.upper, 4)
+
+        m.c = Constraint.Skip
+        self.assertEqual(len(m.c), 0)
+
 class TestArrayCon(unittest.TestCase):
 
     def create_model(self):
@@ -749,7 +769,6 @@ class TestArrayCon(unittest.TestCase):
         return model
 
     def test_rule_option1(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         def f(model, i):
@@ -767,28 +786,7 @@ class TestArrayCon(unittest.TestCase):
         self.assertEqual(model.c[2](), 16)
         self.assertEqual(len(model.c), 4)
 
-    def test_old_rule_option1(self):
-        """Test rule option"""
-        model = self.create_model()
-        buffer = LogBuffer('pyomo.core', logging.WARNING)
-        model.B = RangeSet(1,4)
-        def f(model, i):
-            ans=0
-            for j in model.B:
-                ans = ans + model.x[j]
-            ans *= i
-            ans = ans <= 0
-            ans = ans >= 0
-            return ans
-        model.x = Var(model.B, initialize=2)
-        model.c = Constraint(model.A,rule=f)
-
-        self.assertEqual(model.c[1](), 8)
-        self.assertEqual(model.c[2](), 16)
-        self.assertEqual(len(model.c), 4)
-
     def test_rule_option2(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         def f(model, i):
@@ -808,7 +806,6 @@ class TestArrayCon(unittest.TestCase):
         self.assertEqual(len(model.c), 2)
 
     def test_rule_option3(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         def f(model, i):
@@ -828,7 +825,6 @@ class TestArrayCon(unittest.TestCase):
         self.assertEqual(len(model.c), 2)
 
     def test_rule_option2a(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         @simple_constraint_rule
@@ -849,7 +845,6 @@ class TestArrayCon(unittest.TestCase):
         self.assertEqual(len(model.c), 2)
 
     def test_rule_option3a(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         @simple_constraint_rule
@@ -870,21 +865,18 @@ class TestArrayCon(unittest.TestCase):
         self.assertEqual(len(model.c), 2)
 
     def test_dim(self):
-        """Test dim method"""
         model = self.create_model()
         model.c = Constraint(model.A)
 
         self.assertEqual(model.c.dim(),1)
 
     def test_keys(self):
-        """Test keys method"""
         model = self.create_model()
         model.c = Constraint(model.A)
 
         self.assertEqual(len(list(model.c.keys())),0)
 
     def test_len(self):
-        """Test len method"""
         model = self.create_model()
         model.c = Constraint(model.A)
         self.assertEqual(len(model.c),0)
@@ -903,6 +895,28 @@ class TestArrayCon(unittest.TestCase):
 
         self.assertEqual(len(model.c),1)
 
+    def test_setitem(self):
+        m = ConcreteModel()
+        m.x = Var()
+        m.c = Constraint(range(5))
+        self.assertEqual(len(m.c), 0)
+
+        m.c[2] = m.x**2 <= 4
+        self.assertEqual(len(m.c), 1)
+        self.assertEqual(list(m.c.keys()), [2])
+        self.assertIsInstance(m.c[2], _GeneralConstraintData)
+        self.assertEqual(m.c[2].upper, 4)
+
+        m.c[3] = Constraint.Skip
+        self.assertEqual(len(m.c), 1)
+        self.assertRaisesRegexp( KeyError, "3", m.c.__getitem__, 3)
+
+        self.assertRaisesRegexp( ValueError, "'c\[3\]': rule returned None",
+                                 m.c.__setitem__, 3, None)
+        self.assertEqual(len(m.c), 1)
+
+        m.c[2] = Constraint.Skip
+        self.assertEqual(len(m.c), 0)
 
 class TestConList(unittest.TestCase):
 
@@ -930,7 +944,6 @@ class TestConList(unittest.TestCase):
         self.assertEqual(len(model.c), 1)
 
     def test_rule_option1(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         def f(model, i):
@@ -951,7 +964,6 @@ class TestConList(unittest.TestCase):
         self.assertEqual(len(model.c), 4)
 
     def test_rule_option2(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         def f(model, i):
@@ -972,7 +984,6 @@ class TestConList(unittest.TestCase):
         self.assertEqual(len(model.c), 2)
 
     def test_rule_option1a(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         @simple_constraintlist_rule
@@ -994,7 +1005,6 @@ class TestConList(unittest.TestCase):
         self.assertEqual(len(model.c), 4)
 
     def test_rule_option2a(self):
-        """Test rule option"""
         model = self.create_model()
         model.B = RangeSet(1,4)
         @simple_constraintlist_rule
@@ -1016,7 +1026,6 @@ class TestConList(unittest.TestCase):
         self.assertEqual(len(model.c), 2)
 
     def test_rule_option3(self):
-        """Test rule option"""
         model = self.create_model()
         model.y = Var(initialize=2)
         def f(model):
@@ -1032,7 +1041,6 @@ class TestConList(unittest.TestCase):
         self.assertEqual(model.d[1](), 2)
 
     def test_rule_option4(self):
-        """Test rule option"""
         model = self.create_model()
         model.y = Var(initialize=2)
         model.c = ConstraintList(rule=((i+1)*model.y >= 0 for i in range(3)))
@@ -1040,21 +1048,18 @@ class TestConList(unittest.TestCase):
         self.assertEqual(model.c[1](), 2)
 
     def test_dim(self):
-        """Test dim method"""
         model = self.create_model()
         model.c = ConstraintList()
 
         self.assertEqual(model.c.dim(),1)
 
     def test_keys(self):
-        """Test keys method"""
         model = self.create_model()
         model.c = ConstraintList()
 
         self.assertEqual(len(list(model.c.keys())),0)
 
     def test_len(self):
-        """Test len method"""
         model = self.create_model()
         model.c = ConstraintList()
 
@@ -1086,25 +1091,6 @@ class Test2DArrayCon(unittest.TestCase):
         self.assertEqual(model.c[1,1](), 8)
         self.assertEqual(model.c[2,1](), 16)
 
-    def test_old_rule_option(self):
-        """Test rule option"""
-        model = self.create_model()
-        buffer = LogBuffer('pyomo.core', logging.WARNING)
-        model.B = RangeSet(1,4)
-        def f(model, i, j):
-            ans=0
-            for j in model.B:
-                ans = ans + model.x[j]
-            ans *= i
-            ans = ans <= 0
-            ans = ans >= 0
-            return ans
-        model.x = Var(model.B, initialize=2)
-        model.c = Constraint(model.A,model.A,rule=f)
-
-        self.assertEqual(model.c[1,1](), 8)
-        self.assertEqual(model.c[2,1](), 16)
-
     def test_dim(self):
         """Test dim method"""
         model = self.create_model()
@@ -1149,12 +1135,15 @@ class MiscConTests(unittest.TestCase):
         model.cL = Constraint(expr=model.x**2 >= L)
         self.assertEqual(model.cL.lslack(), 5.0)
         self.assertEqual(model.cL.uslack(), float('inf'))
+        self.assertEqual(model.cL.slack(), 5.0)
         model.cU = Constraint(expr=model.x**2 <= U)
-        self.assertEqual(model.cU.lslack(), float('-inf'))
+        self.assertEqual(model.cU.lslack(), float('inf'))
         self.assertEqual(model.cU.uslack(), 1.0)
+        self.assertEqual(model.cU.slack(), 1.0)
         model.cR = Constraint(expr=L <= model.x**2 <= U)
         self.assertEqual(model.cR.lslack(), 5.0)
         self.assertEqual(model.cR.uslack(), 1.0)
+        self.assertEqual(model.cR.slack(), 1.0)
 
     def test_constructor(self):
         a = Constraint(name="b")
