@@ -1,11 +1,12 @@
-#  _________________________________________________________________________
+#  ___________________________________________________________________________
 #
 #  Pyomo: Python Optimization Modeling Objects
-#  Copyright (c) 2014 Sandia Corporation.
-#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
-#  the U.S. Government retains certain rights in this software.
-#  This software is distributed under the BSD License.
-#  _________________________________________________________________________
+#  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
+#  Under the terms of Contract DE-NA0003525 with National Technology and 
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  rights in this software.
+#  This software is distributed under the 3-clause BSD License.
+#  ___________________________________________________________________________
 #
 # Unit Tests for Elements of a Model
 #
@@ -328,8 +329,8 @@ class TestConnector(unittest.TestCase):
         m.x = Var()
         m.y = Var()
         m.CON = Connector()
-        m.CON.add(-m.x)
-        m.CON.add(1 + m.y)
+        m.CON.add(-m.x, name='expr1')
+        m.CON.add(1 + m.y, name='expr2')
 
         # 2 constraints: one has a connector, the other doesn't.  The
         # former should be deactivated and expanded, the latter should
