@@ -44,8 +44,12 @@ class Test(unittest.TestCase):
 
         # Verify the block attributes
         for idx in ref._data.keys():
-            ref_c = ref[idx].component_map()
-            new_c = new[idx].component_map()
+            # It is important to lookup using ref._data[idx] and not
+            # ref[idx], as the model may not be constructed, and
+            # accessing __getitem__ on an abstract model generates an
+            # exception.
+            ref_c = ref._data[idx].component_map()
+            new_c = new._data[idx].component_map()
             self.assertEqual( sorted(ref_c.keys()), sorted(new_c.keys()) )
             for a in ref_c.keys():
                 self.assertEqual(type(ref_c[a]),  type(new_c[a]))
