@@ -10,20 +10,24 @@
 
 import weakref
 import logging
+import textwrap
 
+import pyomo.util.config as cfg
 from pyomo.util.modeling import unique_component_name
 from pyomo.util.plugin import alias
-from pyomo.core import *
+from pyomo.core import (
+    Block, Connector, Constraint, Param, Set, Suffix, Var,
+    Expression, SortComponents, TraversalStrategy,
+    Any, Reals, NumericConstant, value
+)
 from pyomo.core.base import expr as EXPR, Transformation
-from pyomo.core.base.block import SortComponents
-from pyomo.core.base.component import ComponentUID, ActiveComponent
-from pyomo.core.base import _ExpressionData
+from pyomo.core.base.component import ActiveComponent
 from pyomo.core.base.var import _VarData
-from pyomo.repn import generate_canonical_repn, LinearCanonicalRepn
 from pyomo.core.kernel import ComponentMap, ComponentSet
 from pyomo.core.base.expr import identify_variables
+from pyomo.repn import generate_canonical_repn, LinearCanonicalRepn
 from pyomo.gdp import Disjunct, Disjunction, GDP_Error
-from pyomo.gdp.util import clone_without_expression_components
+from pyomo.gdp.util import clone_without_expression_components, target_list
 from pyomo.gdp.plugins.gdp_var_mover import HACK_GDP_Disjunct_Reclassifier
 
 from six import iteritems, iterkeys
@@ -33,59 +37,126 @@ from nose.tools import set_trace
 
 logger = logging.getLogger('pyomo.gdp.chull')
 
-# NL_Mode_LeeGrossmann is the original NL convex hull from Lee &
-# Grossmann (2000), which substitutes nonlinear constraints
-#
-#     h_ik(x) <= 0
-#
-# with
-#
-#     x_k = sum( nu_ik )
-#     y_ik * h_ik( nu_ik/y_ik ) <= 0
-#
-# [1] Lee, S., & Grossmann, I. E. (2000). New algorithms for nonlinear
-# generalized disjunctive programming.  Computers and Chemical
-# Engineering, 24, 2125-2141
-#
-NL_Mode_LeeGrossmann = 1
-#
-# NL_Mode_GrossmannLee is an updated formulation from Grossmann & Lee
-# (2003), which substitutes nonlinear constraints
-#
-#     h_ik(x) <= 0
-#
-# with
-#
-#     x_k = sum( nu_ik )
-#     (y_ik + eps) * h_ik( nu_ik/(y_ik + eps) ) <= 0
-#
-# [2] Grossmann, I. E., & Lee, S. (2003). Generalized disjunctive
-# programming: Nonlinear convex hull relaxation and algorithms.
-# Computational Optimization and Applications, 26, 83-100.
-#
-NL_Mode_GrossmannLee = 2
-#
-# NL_Mode_FurmanSawayaGrossmann is an improved relaxation that avoids
-# numerical issues from the Lee & Grossmann formulation by using:
-#
-#     x_k = sum( nu_ik )
-#     ((1-eps)*y_ik + eps) * h_ik( nu_ik/((1-eps)*y_ik + eps) ) \
-#        - eps * h_ki(0) * ( 1-y_ik ) <= 0
-#
-# [3] Furman, K., Sawaya, N., and Grossmann, I.  A computationally
-# useful algebraic representation of nonlinear disjunctive convex sets
-# using the perspective function.  Optimization Online (2016).
-#
-# http://www.optimization-online.org/DB_HTML/2016/07/5544.html.
-#
-NL_Mode_FurmanSawayaGrossmann = 3
-
-EPS = 1e-4
 
 class ConvexHull_Transformation(Transformation):
+    """Relax disjunctive model by forming the convex hull.
 
-    alias('gdp.chull', doc="Relaxes a disjunctive model into an algebraic "
-          "model by forming the convex hull relaxation of each disjunction.")
+    Relaxes a disjunctive model into an algebraic model by forming the
+    convex hull of each disjunction.
+
+    This transformation accepts the following keyword arguments:
+
+    Parameters
+    ----------
+    perspective_function : str
+        The perspective function used for the disaggregated variables.
+        Must be one of 'FurmanSawayaGrossmann' (default),
+        'LeeGrossmann', or 'GrossmannLee'
+    EPS : float
+        The value to use for epsilon [default: 1e-4]
+    targets : (block, disjunction, ComponentUID or list of those types)
+        The targets to transform. This can be a block, disjunction, or a
+        list of blocks and Disjunctions [default: the instance]
+
+    After transformation, every transformed disjunct will have a
+    "_gdp_transformation_info" dict containing 2 entries:
+
+        'relaxed': True,
+        'chull': {
+            'relaxationBlock': <block>,
+            'relaxedConstraints': ComponentMap(constraint: relaxed_constraint)
+            'disaggregatedVars': ComponentMap(var: list of disaggregated vars),
+            'bigmConstraints': ComponentMap(disaggregated var: bigM constraint),
+        }
+
+    In addition, any block or disjunct containing a relaxed disjunction
+    will have a "_gdp_transformation_info" dict with the following
+    entry:
+
+        'disjunction_or_constraint': <constraint>
+
+    Finally, the transformation will create a new Block with a unique
+    name beginning "_pyomo_gdp_chull_relaxation".  That Block will
+    contain an indexed Block named "relaxedDisjuncts", which will hold
+    the relaxed disjuncts.  This block is indexed by an integer
+    indicating the order in which the disjuncts were relaxed.  Each
+    block will have a "_gdp_transformation_info" dict with the following
+    entries:
+
+        'src': <source disjunct>
+        'srcVars': ComponentMap(disaggregated var: original var),
+        'srcConstraints': ComponentMap(relaxed_constraint: constraint)
+        'boundConstraintToSrcVar': ComponentMap(bigm_constraint: orig_var),
+
+    """
+
+    alias('gdp.chull', doc=textwrap.fill(textwrap.dedent(__doc__.strip())))
+
+
+    CONFIG = cfg.ConfigBlock('gdp.chull')
+    CONFIG.declare('targets', cfg.ConfigValue(
+        default=None,
+        domain=target_list,
+        description="target or list of targets that will be relaxed",
+        doc="""
+
+        This specifies the target or list of targets to relax as either a
+        component, ComponentUID, or string that can be passed to a
+        ComponentUID; or an iterable of these types.  If None (default),
+        the entire model is transformed."""
+    ))
+    CONFIG.declare('perspective function', cfg.ConfigValue(
+        default='FurmanSawayaGrossmann',
+        domain=cfg.In(['FurmanSawayaGrossmann','LeeGrossmann','GrossmannLee']),
+        description='perspective function used for variable disaggregation',
+        doc="""
+        The perspective function used for variable disaggregation
+
+        "LeeGrossmann" is the original NL convex hull from Lee &
+        Grossmann (2000) [1]_, which substitutes nonlinear constraints
+
+            h_ik(x) <= 0
+
+        with
+
+            x_k = sum( nu_ik )
+            y_ik * h_ik( nu_ik/y_ik ) <= 0
+
+        "GrossmannLee" is an updated formulation from Grossmann &
+        Lee (2003) [2]_, which avoids divide-by-0 errors by using:
+
+            x_k = sum( nu_ik )
+            (y_ik + eps) * h_ik( nu_ik/(y_ik + eps) ) <= 0
+
+        "FurmanSawayaGrossmann" (default) is an improved relaxation [3]_
+        that is exact at 0 and 1 while avoiding numerical issues from
+        the Lee & Grossmann formulation by using:
+
+            x_k = sum( nu_ik )
+            ((1-eps)*y_ik + eps) * h_ik( nu_ik/((1-eps)*y_ik + eps) ) \
+                - eps * h_ki(0) * ( 1-y_ik ) <= 0
+
+        References
+        ----------
+        .. [1] Lee, S., & Grossmann, I. E. (2000). New algorithms for
+           nonlinear generalized disjunctive programming.  Computers and
+           Chemical Engineering, 24, 2125-2141
+
+        .. [2] Grossmann, I. E., & Lee, S. (2003). Generalized disjunctive
+           programming: Nonlinear convex hull relaxation and algorithms.
+           Computational Optimization and Applications, 26, 83-100.
+
+        .. [3] Furman, K., Sawaya, N., and Grossmann, I.  A computationally
+           useful algebraic representation of nonlinear disjunctive convex
+           sets using the perspective function.  Optimization Online
+           (2016). http://www.optimization-online.org/DB_HTML/2016/07/5544.html.
+        """
+    ))
+    CONFIG.declare('EPS', cfg.ConfigValue(
+        default=1e-4,
+        domain=cfg.PositiveFloat,
+        description="Epsilon value to use in perspective function",
+    ))
 
     def __init__(self):
         super(ConvexHull_Transformation, self).__init__()
@@ -93,6 +164,7 @@ class ConvexHull_Transformation(Transformation):
             Constraint : self._xform_constraint,
             Var :        False,
             Connector :  False,
+            Expression : False,
             Param :      False,
             Set :        False,
             Suffix :     False,
@@ -101,24 +173,10 @@ class ConvexHull_Transformation(Transformation):
             Block:       self._transform_block_on_disjunct,
             }
 
-        #self._mode = NL_Mode_LeeGrossmann
-        #self._mode = NL_Mode_GrossmannLee
-        self._mode = NL_Mode_FurmanSawayaGrossmann
-
 
     def _apply_to(self, instance, **kwds):
-        options = kwds.pop('options', {})
-        targets = kwds.pop('targets', None)
-        self._mode = kwds.pop('nl_mode', NL_Mode_FurmanSawayaGrossmann)
-
-        if kwds:
-            logger.warning("GDP(CHull): unrecognized keyword arguments:\n\t%s"
-                           % ( '\n\t'.join(iterkeys(kwds)), ))
-
-        # We don't accept any options at the moment.
-        if options:
-            logger.warning("GDP(CHull): unrecognized options:\n%s"
-                        % ( '\n'.join(iterkeys(options)), ))
+        self._config = self.CONFIG(kwds.pop('options', {}))
+        self._config.set_value(kwds)
 
         # make a transformation block
         transBlockName = unique_component_name(
@@ -130,6 +188,7 @@ class ConvexHull_Transformation(Transformation):
         transBlock.lbub = Set(initialize = ['lb','ub','eq'])
         transBlock.disjContainers = ComponentSet()
 
+        targets = self._config.targets
         if targets is None:
             targets = ( instance, )
             _HACK_transform_whole_instance = True
@@ -140,8 +199,6 @@ class ConvexHull_Transformation(Transformation):
             if t is None:
                 raise GDP_Error(
                     "Target %s is not a component on the instance!" % _t)
-            if not t.active:
-                continue
 
             if t.type() is Disjunction:
                 if t.parent_component() is t:
@@ -319,22 +376,34 @@ class ConvexHull_Transformation(Transformation):
         varOrder = []
         varsByDisjunct = ComponentMap()
         for disjunct in obj.disjuncts:
-            disjunctVars = varsByDisjunct[disjunct] = ComponentSet()
-            for cons in disjunct.component_data_objects(
-                    Constraint,
-                    active = True,
-                    sort=SortComponents.deterministic,
-                    descend_into=Block):
-                # we aren't going to disaggregate fixed variables. This
-                # means there is trouble if they are unfixed later...
-                for var in identify_variables(cons.body, include_fixed=False):
-                    # Note the use of a list so that we will eventually
-                    # disaggregate the vars in a deterministic order
-                    # (the order that we found them)
-                    disjunctVars.add(var)
-                    if var not in varOrder_set:
-                        varOrder.append(var)
-                        varOrder_set.add(var)
+            # This is crazy, but if the disjunct has been previously
+            # relaxed, the disjunct *could* be deactivated.
+            not_active = not disjunct.active
+            if not_active:
+                disjunct._activate_without_unfixing_indicator()
+            try:
+                disjunctVars = varsByDisjunct[disjunct] = ComponentSet()
+                for cons in disjunct.component_data_objects(
+                        Constraint,
+                        active = True,
+                        sort=SortComponents.deterministic,
+                        descend_into=Block):
+                    # we aren't going to disaggregate fixed
+                    # variables. This means there is trouble if they are
+                    # unfixed later...
+                    for var in identify_variables(
+                            cons.body, include_fixed=False):
+                        # Note the use of a list so that we will
+                        # eventually disaggregate the vars in a
+                        # deterministic order (the order that we found
+                        # them)
+                        disjunctVars.add(var)
+                        if var not in varOrder_set:
+                            varOrder.append(var)
+                            varOrder_set.add(var)
+            finally:
+                if not_active:
+                    disjunct._deactivate_without_fixing_indicator()
 
         # We will only disaggregate variables that
         #  1) appear in multiple disjuncts, or
@@ -654,20 +723,21 @@ class ConvexHull_Transformation(Transformation):
             c = obj[i]
             if not c.active:
                 continue
-            c.deactivate()
 
             NL = c.body.polynomial_degree() not in (0,1)
+            EPS = self._config.EPS
+            mode = self._config.perspective_function
 
             # We need to evaluate the expression at the origin *before*
             # we substitute the expression variables with the
             # disaggregated variables
-            if not NL or self._mode == NL_Mode_FurmanSawayaGrossmann:
+            if not NL or mode == "FurmanSawayaGrossmann":
                 h_0 = clone_without_expression_components(
                     c.body, substitute=zero_substitute_map)
 
             y = disjunct.indicator_var
             if NL:
-                if self._mode == NL_Mode_LeeGrossmann:
+                if mode == "LeeGrossmann":
                     sub_expr = clone_without_expression_components(
                         c.body,
                         substitute=dict(
@@ -675,7 +745,7 @@ class ConvexHull_Transformation(Transformation):
                             for var, subs in iteritems(var_substitute_map) )
                     )
                     expr = sub_expr * y
-                elif self._mode == NL_Mode_GrossmannLee:
+                elif mode == "GrossmannLee":
                     sub_expr = clone_without_expression_components(
                         c.body,
                         substitute=dict(
@@ -683,7 +753,7 @@ class ConvexHull_Transformation(Transformation):
                             for var, subs in iteritems(var_substitute_map) )
                     )
                     expr = (y + EPS) * sub_expr
-                elif self._mode == NL_Mode_FurmanSawayaGrossmann:
+                elif mode == "FurmanSawayaGrossmann":
                     sub_expr = clone_without_expression_components(
                         c.body,
                         substitute=dict(
