@@ -13,7 +13,7 @@ import logging
 import copy
 
 import pyomo.util.plugin
-from pyomo.opt import SolverFactory, PersistentSolver
+from pyomo.opt import SolverFactory
 from pyomo.core import *
 from pyomo.pysp import phextension
 from pyomo.pysp.plugins.phboundextension import (_PHBoundBase,
@@ -221,6 +221,10 @@ class convexhullboundextension(pyomo.util.plugin.SingletonPlugin, _PHBoundBase):
     # populate the master bundle model from the PH parameters
     #
     def _populate_bundle_dual_master_model(self, ph):
+        # TODO: Does this import need to be delayed because
+        #       it is in a plugins subdirectory
+        from pyomo.solvers.plugins.solvers.persistent_solver import \
+            PersistentSolver
 
         current_iteration = ph._current_iteration
 
@@ -272,7 +276,7 @@ class convexhullboundextension(pyomo.util.plugin.SingletonPlugin, _PHBoundBase):
             # we can test the script and not have to worry
             # about a missing solver
             if isinstance(solver, PersistentSolver):
-                solver.compile_instance(self._master_model)
+                solver.set_instance(self._master_model)
             results = solver.solve(self._master_model)
             self._master_model.solutions.load_from(results)
 #        print "MASTER MODEL WVAR FOLLOWING SOLVE:"
