@@ -10,7 +10,7 @@
 
 import pyomo.util.plugin
 
-from six import iteritems, itervalues
+from six import iteritems, itervalues, print_
 
 import random
 
@@ -51,7 +51,7 @@ class EcksteinCombettesExtension(pyomo.util.plugin.SingletonPlugin):
         random.seed(1234)
         print("Kludge warning: set random seed to 1234")
 
-        self._check_output = True
+        self._check_output = False
         self._JName = "PhiSummary.csv"
 
         self._subproblems_to_queue = []
@@ -209,15 +209,12 @@ class EcksteinCombettesExtension(pyomo.util.plugin.SingletonPlugin):
             for tree_node in scenario._node_list[:-1]:
                 tree_node_zs = tree_node._z
                 for variable_id in tree_node._standard_variable_ids:
-                        var_values = scenario._x[tree_node._name]
-                        varval = var_values[variable_id]
-                        weight_values = scenario._w[tree_node._name]
-                        if varval is not None:
-                            this_sub_phi_term = scenario._probability * ((tree_node_zs[variable_id] - varval) * (scenario._y[variable_id] + weight_values[variable_id]))
-                            cumulative_sub_phi += this_sub_phi_term
-                            
-                        else:
-                            foobar # HEY! Fix this!
+                    var_values = scenario._x[tree_node._name]
+                    varval = var_values[variable_id]
+                    weight_values = scenario._w[tree_node._name]
+                    if not scenario.is_variable_stale(tree_node, variable_id):
+                        this_sub_phi_term = scenario._probability * ((tree_node_zs[variable_id] - varval) * (scenario._y[variable_id] + weight_values[variable_id]))
+                        cumulative_sub_phi += this_sub_phi_term
 
             with open(self._JName,"a") as f:
                 f.write(", %10f" % (cumulative_sub_phi))
@@ -234,6 +231,7 @@ class EcksteinCombettesExtension(pyomo.util.plugin.SingletonPlugin):
         for scenario_name in sorted(sub_phi_map.keys()):
             print("  %30s %16e" % (scenario_name, sub_phi_map[scenario_name]))
 
+        print("")
         print("Computed phi:    %16e" % phi)
         if phi > 0:
             tau = 1.0 # this is the over-relaxation parameter - we need to do something more useful
@@ -346,11 +344,9 @@ class EcksteinCombettesExtension(pyomo.util.plugin.SingletonPlugin):
                     var_values = scenario._x[tree_node._name]
                     varval = var_values[variable_id]
                     weight_values = scenario._w[tree_node._name]
-                    if varval is not None:
+                    if not scenario.is_variable_stale(tree_node, variable_id):
                         this_sub_phi_term = scenario._probability * ((tree_node_zs[variable_id] - varval) * (scenario._y[variable_id] + weight_values[variable_id]))
                         cumulative_sub_phi += this_sub_phi_term
-                    else:
-                        foobar # HEY! Fix this!
 
             with open(self._JName,"a") as f:
                 f.write(", %10f" % (cumulative_sub_phi))
@@ -363,11 +359,12 @@ class EcksteinCombettesExtension(pyomo.util.plugin.SingletonPlugin):
 
         print("Computed sub-phi values (scenario, phi, iters-since-last-incorporated):")
         for sub_phi in sorted(sub_phi_to_scenario_map.keys()):
-            print("  %16e: " % sub_phi),
+            print_("  %16e: " % sub_phi, end="")
             for scenario_name in sub_phi_to_scenario_map[sub_phi]:
-                print("%30s" % scenario_name),
-                print(" %4d" % (self._total_projection_steps - self._projection_step_of_last_update[scenario_name])), # TBD - not handling multiple scenarios correctly here
-            print("")
+                print("%30s %4d" % (scenario_name,
+                                    self._total_projection_steps - self._projection_step_of_last_update[scenario_name]))
+
+        print("")
 
         print("Computed phi: %16e" % phi)
         with open(self._JName,"a") as f:
@@ -394,9 +391,8 @@ class EcksteinCombettesExtension(pyomo.util.plugin.SingletonPlugin):
             for phi in sorted_phis[0:ph._async_buffer_length]:
                 if ((self._queue_only_negative_subphi_subproblems) and (phi < 0.0)) or (not self._queue_only_negative_subphi_subproblems):
                     scenario_name = sub_phi_to_scenario_map[phi][0] 
-                    print("%30s %16e" % (scenario_name,phi)),
+                    print_("%30s %16e" % (scenario_name,phi), end="")
                     self._subproblems_to_queue.append(scenario_name)
-                    print("")
 
         print("")
 
