@@ -18,6 +18,8 @@ from pyomo.pysp.scenariotree.instance_factory import \
     ScenarioTreeInstanceFactory
 from pyomo.pysp.scenariotree.tree_structure_model import \
     CreateAbstractScenarioTreeModel
+from pyomo.pysp.scenariotree.tree_structure import \
+    ScenarioTree
 from pyomo.pysp.util.misc import load_external_module
 
 try:
@@ -60,6 +62,8 @@ class Test(unittest.TestCase):
             del sys.modules["reference_test_scenario_tree_model"]
         if "ScenarioStructure" in sys.modules:
             del sys.modules["ScenarioStructure"]
+        if "both_callbacks" in sys.modules:
+            del sys.modules["both_callbacks"]
 
     def _get_testfname_prefix(self):
         class_name, test_name = self.id().split('.')[-2:]
@@ -133,6 +137,8 @@ class Test(unittest.TestCase):
                     bundles="bundles.notexists",
                     verbose=True)
 
+    # model: name of .py file with model
+    # scenario_tree: name of .dat file
     def test_init1(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -146,6 +152,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: name of directory with ReferenceModel.py with model
+    # scenario_tree: name of directory with ScenarioStructure.dat
     def test_init1_default(self):
         self.assertTrue("ReferenceModel" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -157,6 +165,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("ReferenceModel" in sys.modules)
 
+    # model: name of archive with .py file with model
+    # scenario_tree: name of .dat file
     def test_init2(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -170,6 +180,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: name of archive containing ReferenceModel.py with model
+    # scenario_tree: name of directory with ScenarioStructure.dat file
     def test_init2_default(self):
         self.assertTrue("ReferenceModel" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -181,6 +193,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("ReferenceModel" in sys.modules)
 
+    # model: name of .py file with model
+    # scenario_tree: name of archive with .dat file
     def test_init3(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -194,6 +208,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: name of directory with ReferenceModel.py file with model
+    # scenario_tree: name of archive with ScenarioStructure.dat file
     def test_init3_default(self):
         self.assertTrue("ReferenceModel" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -206,6 +222,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("ReferenceModel" in sys.modules)
 
+    # model: name of archive with .py file with model
+    # scenario_tree: name of copy of archive with .dat file
     def test_init4(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         archive_copy = self._get_testfname_prefix()+".archive_copy.tgz"
@@ -224,6 +242,8 @@ class Test(unittest.TestCase):
         os.remove(archive_copy)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: name of archive with ReferenceModel.py file with model
+    # scenario_tree: name of copy of archive with ScenarioStructure.dat file
     def test_init4_default(self):
         self.assertTrue("ReferenceModel" not in sys.modules)
         archive_copy = self._get_testfname_prefix()+".archive_copy.tgz"
@@ -241,6 +261,8 @@ class Test(unittest.TestCase):
         os.remove(archive_copy)
         self.assertTrue("ReferenceModel" in sys.modules)
 
+    # model: name of archive with .py file with model
+    # scenario_tree: name of same archive with .dat file
     def test_init5(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -255,6 +277,8 @@ class Test(unittest.TestCase):
         self.assertEqual(len(factory._archives), 0)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: name of archive with ReferenceModel.py file with model
+    # scenario_tree: name of same archive with ScenarioStructure.dat file
     def test_init5_default(self):
         self.assertTrue("ReferenceModel" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -267,6 +291,8 @@ class Test(unittest.TestCase):
         self.assertEqual(len(factory._archives), 0)
         self.assertTrue("ReferenceModel" in sys.modules)
 
+    # model: name of .py file with model
+    # scenario_tree: Pyomo scenario tree model
     def test_init6(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         scenario_tree_model = CreateAbstractScenarioTreeModel().\
@@ -306,6 +332,8 @@ class Test(unittest.TestCase):
         self.assertEqual(len(factory._archives), 0)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: Pyomo model
+    # scenario_tree: Pyomo scenario tree model
     def test_init7(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         scenario_tree_model = CreateAbstractScenarioTreeModel().\
@@ -333,6 +361,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertEqual(len(factory._archives), 0)
 
+    # model: Pyomo model
+    # scenario_tree: name of .py file with Pyomo scenario tree model
     def test_init8(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         self.assertTrue("reference_test_scenario_tree_model" not in sys.modules)
@@ -350,6 +380,8 @@ class Test(unittest.TestCase):
         self.assertTrue("reference_test_model" not in sys.modules)
         self.assertTrue("reference_test_scenario_tree_model" in sys.modules)
 
+    # model: name of .py file with model callback
+    # scenario_tree: name of .dat file
     def test_init9(self):
         self.assertTrue("reference_test_model_with_callback" not in sys.modules)
         with ScenarioTreeInstanceFactory(
@@ -364,6 +396,9 @@ class Test(unittest.TestCase):
         self.assertEqual(len(factory._archives), 0)
         self.assertTrue("reference_test_model_with_callback" in sys.modules)
 
+    # model: name of directory with ReferenceModel.py file with model
+    # scenario_tree: name of .dat file
+    # data: name of directory with yaml files
     @unittest.skipIf(not has_yaml, "PyYAML is not available")
     def test_init10(self):
         with ScenarioTreeInstanceFactory(
@@ -379,6 +414,9 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertEqual(len(factory._archives), 0)
 
+    # model: Pyomo model
+    # scenario_tree: Pyomo scenario tree model
+    # data: name of directory with .dat files
     def test_init11(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         scenario_tree_model = CreateAbstractScenarioTreeModel().\
@@ -395,12 +433,15 @@ class Test(unittest.TestCase):
         self.assertEqual(factory._closed, True)
         self.assertEqual(len(factory._archives), 0)
 
+    # model: model callback
+    # scenario_tree: Pyomo scenario tree model
     def test_init12(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         scenario_tree_model = CreateAbstractScenarioTreeModel().\
             create_instance(
                 join(testdatadir, "reference_test_scenario_tree.dat"))
-        def scenario_model_callback(scenario_name, node_list):
+        def scenario_model_callback(scenario_tree, scenario_name, node_list):
+            self.assertIs(scenario_tree, scenario_tree_model)
             instance = reference_test_model.create_instance()
             if scenario_name == "s1":
                 instance.p = 1.0
@@ -419,9 +460,9 @@ class Test(unittest.TestCase):
         self.assertEqual(factory._closed, True)
         self.assertEqual(len(factory._archives), 0)
 
+    # model: Pyomo model
+    # scenario_tree: Pyomo scenario tree model
     def test_init13(self):
-        # A concrete model and scenario tree model without any data
-        # (all scenarios will be duplicates of the reference model)
         model = reference_test_model.create_instance()
         scenario_tree_model = CreateAbstractScenarioTreeModel().\
             create_instance(
@@ -444,6 +485,9 @@ class Test(unittest.TestCase):
         self.assertEqual(factory._closed, True)
         self.assertEqual(len(factory._archives), 0)
 
+    # model: Pyomo model
+    # scenario_tree: Pyomo scenario tree model
+    # data: name of a directory with .dat files
     def test_init14(self):
         self.assertTrue("reference_test_model" not in sys.modules)
         scenario_tree_model = CreateAbstractScenarioTreeModel().\
@@ -465,6 +509,9 @@ class Test(unittest.TestCase):
         self.assertEqual(factory._closed, True)
         self.assertEqual(len(factory._archives), 0)
 
+    # model: name of a .py file with model
+    # scenario_tree: name of a .py file
+    # data: name of a directory with .dat files
     @unittest.skipIf(not has_networkx, "Requires networkx module")
     def test_init15(self):
         self.assertTrue("reference_test_model" not in sys.modules)
@@ -479,6 +526,8 @@ class Test(unittest.TestCase):
             self._check_factory(factory)
         self.assertTrue("reference_test_model" in sys.modules)
 
+    # model: name of a .py file with model
+    # scenario_tree: networkx tree
     @unittest.skipIf(not has_networkx, "Requires networkx module")
     def test_init16(self):
         self.assertTrue("reference_test_model" not in sys.modules)
@@ -496,6 +545,8 @@ class Test(unittest.TestCase):
         self.assertTrue("reference_test_model" in sys.modules)
         self.assertTrue("ScenarioStructure" in sys.modules)
 
+    # model: name of a .py file with model
+    # scenario_tree: networkx tree
     @unittest.skipIf(not has_networkx, "Requires networkx module")
     def test_init17(self):
         self.assertTrue("reference_test_model" not in sys.modules)
@@ -536,7 +587,120 @@ class Test(unittest.TestCase):
         self.assertTrue("reference_test_model" in sys.modules)
         self.assertTrue("ScenarioStructure" in sys.modules)
 
+    # model: model callback
+    # scenario_tree: networkx tree
+    @unittest.skipIf(not has_networkx, "Requires networkx module")
+    def test_init18(self):
+        self.assertTrue("reference_test_model" not in sys.modules)
+        self.assertTrue("ScenarioStructure" not in sys.modules)
+        nx_tree = load_external_module(os.path.join(testdatadir,
+                                                    "ScenarioStructure.py"))[0].G
+        def scenario_model_callback(scenario_tree, scenario_name, node_list):
+            self.assertIs(scenario_tree, nx_tree)
+            instance = reference_test_model.create_instance()
+            if scenario_name == "s1":
+                instance.p = 1.0
+            elif scenario_name == "s2":
+                instance.p = 2.0
+            else:
+                assert scenario_name == "s3"
+                instance.p = 3.0
+            return instance
+        with ScenarioTreeInstanceFactory(
+                model=scenario_model_callback,
+                scenario_tree=nx_tree) as factory:
+            self.assertTrue(factory.model_directory() is None)
+            self.assertTrue(factory.scenario_tree_directory() is None)
+            self._check_factory(factory)
+        self.assertEqual(factory._closed, True)
+        self.assertEqual(len(factory._archives), 0)
 
+    # model: model callback
+    # scenario_tree: name of .dat file
+    def test_init19(self):
+        self.assertTrue("reference_test_model" not in sys.modules)
+        def scenario_model_callback(scenario_tree, scenario_name, node_list):
+            self.assertTrue(isinstance(scenario_tree, ScenarioTree))
+            instance = reference_test_model.create_instance()
+            if scenario_name == "s1":
+                instance.p = 1.0
+            elif scenario_name == "s2":
+                instance.p = 2.0
+            else:
+                assert scenario_name == "s3"
+                instance.p = 3.0
+            return instance
+        with ScenarioTreeInstanceFactory(
+                model=scenario_model_callback,
+                scenario_tree=join(testdatadir,
+                                   "reference_test_scenario_tree.dat")) as factory:
+            self.assertTrue(factory.model_directory() is None)
+            self.assertTrue(factory.scenario_tree_directory() is not None)
+            self._check_factory(factory)
+        self.assertEqual(factory._closed, True)
+        self.assertEqual(len(factory._archives), 0)
+
+    # model: model callback
+    # scenario_tree: PySP scenario tree
+    def test_init20(self):
+        self.assertTrue("reference_test_model" not in sys.modules)
+        def scenario_model_callback(scenario_tree, scenario_name, node_list):
+            self.assertTrue(isinstance(scenario_tree, ScenarioTree))
+            instance = reference_test_model.create_instance()
+            if scenario_name == "s1":
+                instance.p = 1.0
+            elif scenario_name == "s2":
+                instance.p = 2.0
+            else:
+                assert scenario_name == "s3"
+                instance.p = 3.0
+            return instance
+        with ScenarioTreeInstanceFactory(
+                model=scenario_model_callback,
+                scenario_tree=join(testdatadir,
+                                   "reference_test_scenario_tree.dat")) as factory:
+            my_scenario_tree = factory.generate_scenario_tree()
+        with ScenarioTreeInstanceFactory(
+                model=scenario_model_callback,
+                scenario_tree=my_scenario_tree) as factory:
+            self.assertTrue(factory.model_directory() is None)
+            self.assertTrue(factory.scenario_tree_directory() is None)
+            self._check_factory(factory)
+        self.assertEqual(factory._closed, True)
+        self.assertEqual(len(factory._archives), 0)
+
+    # model: name of .py file with model callback
+    # scenario_tree: name of same .py file with scenario tree callback
+    @unittest.skipIf(not has_networkx, "Requires networkx module")
+    def test_init21(self):
+        self.assertTrue("both_callbacks" not in sys.modules)
+        with ScenarioTreeInstanceFactory(
+                model=join(testdatadir,
+                           "both_callbacks.py"),
+                scenario_tree=join(testdatadir,
+                                   "both_callbacks.py")) as factory:
+            self.assertEqual(len(factory._archives), 0)
+            self.assertTrue(factory.model_directory() is not None)
+            self.assertTrue(factory.scenario_tree_directory() is not None)
+            self._check_factory(factory)
+        self.assertEqual(len(factory._archives), 0)
+        self.assertTrue("both_callbacks" in sys.modules)
+
+    # model: name of .py file with model callback
+    # scenario_tree: None (which implies the model filename)
+    @unittest.skipIf(not has_networkx, "Requires networkx module")
+    def test_init22(self):
+        self.assertTrue("both_callbacks" not in sys.modules)
+        with ScenarioTreeInstanceFactory(
+                model=join(testdatadir,
+                           "both_callbacks.py"),
+                scenario_tree=None) as factory:
+            self.assertEqual(len(factory._archives), 0)
+            self.assertTrue(factory.model_directory() is not None)
+            self.assertTrue(factory.scenario_tree_directory() is not None)
+            self._check_factory(factory)
+        self.assertEqual(len(factory._archives), 0)
+        self.assertTrue("both_callbacks" in sys.modules)
 
 Test = unittest.category('smoke','nightly','expensive')(Test)
 
