@@ -429,7 +429,6 @@ class PersistentSolver(DirectOrPersistentSolver):
                                 self._default_variable_value
                             if self._load_solutions:
                                 _model.load_solution(result.solution(0))
-                                result.solution.clear()
                         else:
                             assert len(result.solution) == 0
                         # see the hack in the write method
@@ -438,6 +437,9 @@ class PersistentSolver(DirectOrPersistentSolver):
                         assert len(getattr(_model, "._symbol_maps")) == 1
                         delattr(_model, "._symbol_maps")
                         del result._smap_id
+                        if self._load_solutions and \
+                           (len(result.solution) == 0):
+                            logger.error("No solution is available")
                     else:
                         if self._load_solutions:
                             _model.solutions.load_from(
